#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 16 16:20:36 2023
@author: arb
BY-NC 4.0
"""

import pyxel
import machine as M
import ruban as R

class Jeu :
    """Pour simuler le fonctionnement d'une machine de Turing"""
    def __init__(self, ruban:str, regles:dict):
        """
        On initialise avec le ruban et les règles ('A' est l'état initial et 'F' l'état final cf. ligne 22)
        """
        pyxel.init(256, 64, title="Turing")
        pyxel.load("turing.pyxres")
        
        machine = M.Machine('A', 'F', regles) #création de la machine 
        self.rubanO = ruban #Le ruban avant de démarrer la machine (de type str)
        self.conf = M.Configuration(machine, self.rubanO) #une instance de la classe Configuration
        
        pyxel.run(self.update, self.draw)
            
    def update(self):
        """
        Mise à jour de la configuration toutes les secondes
        La machine s'arrete lorsque l'état de la configuration est l'état final de la machine ('F' ligne 22)
        """
        pass
        
    def draw(self):
        pyxel.cls(0)
        dicoNB = {'1' : 0 , '0' : 8} #pour retrouver les abscisses des dessins des bits dans le fichier ressource
        dicoLETTRES = {'A' : 0, 'B' : 8, 'C' : 16, 'D' : 24, 'E' : 32, 'F' : 40} #pour retrouver les abscisses des dessins des lettres dans le fichier ressource
        
        pyxel.blt(225, 0, 0, 0, 40, 30, 30, 0) #un peu de décoration
        
        #affichage du ruban initial
        pyxel.text(0, 0, 'Etat du ruban initial :', 8)
        for i in range (len(self.rubanO)):
            if self.rubanO[i] != ' ': #n'affiche rien si case vide
                pyxel.blt( 8*i, 8, 0, dicoNB[self.rubanO[i]], 8, 8, 8, 0 )
        pyxel.blt(0, 8, 0, 0, 0, 8, 8, 0) #position de départ de la machine
        
        #affichage de la configuration actuelle  
        for x in range(-16, 16): #33 cases du ruban affichées. La tête de lecture est positionnée sur la case 0
            val = self.conf.ruban[self.conf.position + x] #la valeur à afficher sur la case x
            if not (val is None) : #on n'affiche rien si val vaut None
                #On affiche la valeur à l'abscisse 128 + 8*x
                
        
        
               
        
        for x in range(0, 255, 8): #affiche des lignes bleues
            pyxel.blt(x, 55, 0, 8, 24, 8, 8, 0)    
            
        pyxel.blt(128, 55, 0, 0, 0, 8, 8, 0) #affichage de la tête de lecture
        pyxel.blt(128, 47, 0, dicoLETTRES[self.conf.etat], 16, 8, 8, 0) #affichage de l'état   
        
        
        
if __name__ == '__main__':  
    regles = { 'A':{ '0':('0', 0, 'B'), '1':('1', 0, 'B'), None : (None, 1, 'A') }, \
              'B' : {'0' : ('1', 1, 'B'), '1' : ('0', 1, 'B'), None : (None, 0, 'F')}}
                           
    ruban = '  010011  110' #le ruban
    Jeu(ruban, regles)