#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 16 14:18:18 2023
@author: arb
BY-NC 4.0
"""

import ruban as R

class Machine :
    """
    Une Machine possède 3 attributs : sont état de départ (str), son état final (str) et ses règles (dict) 
    """
    def __init__ (self, depart, final, regles):
        self.etat_initial = depart
        self.etat_final = final
        self.regles = regles
        
class Configuration :
    """
    Une Configuration possède 4 attributs :
        La Machine à qui elle est associée
        son état (à l'intialisation son état est l'état initial de la Machine)
        son ruban (un objet de la classe Ruban)
        sa position sur le ruban (à l'initialisation la position vaut 0')
    """
    
    def __init__(self, machine, ruban:str):
        """
        machine : un objet de la classe Machine à qui est associée la configuration
        ruban : une chaîne de caractère. Les cases vides sont implémentées par un espace
        """
        self.machine = machine
        self.etat = machine.etat_initial
        self.ruban = R.Ruban()
        for element in ruban :
            if element == ' ':
                self.ruban.append(None)
            else :
                self.ruban.append(element)
        self.position = 0
        
    def evolue(self)->None:
        """
        Pour faire évoluer (un pas) la configuration selon les règles de la machine

        """
        position = self.position
        etat = self.etat
        valeur = self.ruban[position]
        #self.ruban[position] = ...
        #self.position = ...
        
        if self.position > self.ruban.imax() :
            self.ruban.append(None)
        if self.position < self.ruban.imin():
            self.ruban.prepend(None)
            
        #self.etat = ...
        
    def __str__(self)->str:
        """Affichage de la configuration. L'état actuel est indiqué entre paranthésesMachine devant sa position"""
        s = ''
        for i in range(self.ruban.imin(), self.ruban.imax() + 1) :
            if self.ruban[i] is None :
                val = ''
            else :
                val = self.ruban[i]
            if self.position == i :
                s  = s + '(' + self.etat + ')' + ' ' + val + ' '
           
            else :
                s = s + val + ' '
        return s
                
                
if __name__ == '__main__':
    #les règles
    regles = { 'A':{ '0':('0', 0, 'B'), '1':('1', 0, 'B'), None : (None, 1, 'A') }, \
              'B' : {'0' : ('1', 1, 'B'), '1' : ('0', 1, 'B'), None : (None, 0, 'F')}}
                           
    ruban = '  010011  110' #le ruban
    machine = Machine('A', 'F', regles) #création de la machine
    conf = Configuration(machine, ruban) #création de la configuration
    print(conf)
         
    while conf.etat != conf.machine.etat_final : #Tant que l'état de la configuration n'est pas l'état final de la machine
        conf.evolue() #On fait évoluer la configuration (1 pas)
        print(conf)
    
    