#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 27 18:14:53 2021

@author: a.rb
"""

class Ruban:
    """
    Une classe pour les tableaux bidirectionnels
    Ces tableaux sont indexés de -n à +m 
    Attention, par exemple, la valeur stockée à l'indice -1 n'est donc pas la dernière valeur du tableau mais celle stockée à gauche de la valeur d'indice 0'
    """
    def __init__(self):
        """
        A l'initialisation un ruban ne prend aucun argument , il s'agit d'un ruban vide
        >>> ruban = Ruban()
         
        On utilisera ensuite les méthode append et prepend pour écrire sur le ruban

        """
        self.gauche = []
        self.droite = []
        
    def imin(self):
        """retourne l'indice le plus petit du tableau (peut être négatif)"""
        return(-len(self.gauche))
        
    def imax(self):
        """retourne l'indice le plus grand du tableau"""
        return(len(self.droite) - 1)
        
    def append(self, v):
        """
        permet d'ajouter un élèment à droite
        """
        self.droite.append(v)
        
    def prepend(self, v):
        """
        permet d'ajouter un élément à gauche
        """
        self.gauche.append(v)
        
    def __getitem__(self, i):
        """
        permet d'accéder aux valeurs du tableau avec la syntaxe tab[i]
        
        si l'indice dépasse (à droite ou à gauche) les indices imin ou imax, cette fonction retourne None
        """
        if i > self.imax() or i < self.imin():
            return None
        if i >= 0 :
            return self.droite[i]
        else :
            return self.gauche[-i - 1]
        
    def __setitem__(self, i, v):
        """
        permet de modifier la valeur d'un élément d'indice i avec la syntaxe tab[i]
        Attention i doit être dans l'intervalle [imin ; imax]
        """

        if i >= 0:
            self.droite[i] = v
        else :
            self.gauche[-i - 1] = v
            
    def __str__(self):
        """affichage du tableau"""
        return str(self.gauche[::-1])[0:-1] + ',' + str(self.droite)[1:] + ' imin :' + str(self.imin()) + ' imax :' + str(self.imax())
    
if __name__ == '__main__':
    ruban = Ruban() #ruban vide
    for element in '10011' : #ajout des élèments à partir d'une chaîne de caratères
        ruban.append(element) #les élèments sont indexés à partir de 0
    print('ruban :', ruban) #affichage du tableau
    
    ruban.prepend('0') #ajout d'une valeur à gauche
    print('ruban :', ruban)
    print('ruban[-1]:', ruban[-1]) #affichage de la valeur à gauche de celle d'indice 0
    print('ruban[-2]:', ruban[-2]) #cette valeur n'existe pas
    print('ruban[100]:', ruban[100]) #cette valeur n'existe pas
    
    ruban[-1] = '1' #modification de la valeur d'indice -1
    print('ruban :', ruban)
 
